package edu.odu.cs.codeAnnotation;

import static org.junit.Assert.assertFalse;
import static org.junit.Assert.assertTrue;

import java.io.BufferedReader;
import java.io.FileReader;
import java.io.IOException;

import org.junit.Test;

public class TestC2HOptions {
    
    public String load(String fileName) throws IOException {
        StringBuffer results = new StringBuffer();
        BufferedReader in = new BufferedReader(new FileReader(fileName));
        String line = in.readLine();
        while (line != null) {
            results.append(line);
            results.append("\n");
            line = in.readLine();
        }
        in.close();
        return results.toString();
    }

    @Test
    public void testMain() throws Exception {
        String[] args = {"test.cpp", "test.cpp.html"};
        Code2HTML.main(args);
        String out = load("test.cpp.html");
        assertTrue (out.contains("a line started by 8 blanks"));
        assertTrue (out.contains("<style"));
        assertTrue (out.contains("<title>test.cpp</title>"));
        assertTrue (out.contains("<h1>test.cpp</h1>"));
    }
    
    @Test
    public void testCSS() throws Exception {
        String[] args = {"-css", "other.css", "test.cpp", "test.cpp.html"};
        Code2HTML.main(args);
        String out = load("test.cpp.html");
        assertTrue (out.contains("a line started by 8 blanks"));
        assertTrue (out.contains("other.css"));
    }

    @Test
    public void testEmbedCSS() throws Exception {
        String[] args = {"-css", "code2html.css", "-embedcss", "test.cpp", "test.cpp.html"};
        Code2HTML.main(args);
        String out = load("test.cpp.html");
        assertTrue (out.contains("a line started by 8 blanks"));
        assertTrue (out.contains("monospace"));
        assertTrue (out.contains("blue"));
    }

    @Test
    public void testEmbedCSSDefault() throws Exception {
        String[] args = {"-embedcss", "test.cpp", "test.cpp.html"};
        Code2HTML.main(args);
        String out = load("test.cpp.html");
        assertTrue (out.contains("a line started by 8 blanks"));
        assertTrue (out.contains("italic"));
        assertFalse (out.contains("blue"));
    }
    
    @Test
    public void testFooter() throws Exception {
        String[] args = {"-footer", "<p>something at the bottom</p>", "test.cpp"};
        Code2HTML.main(args);
        String out = load("test.cpp.html");
        assertTrue (out.contains("a line started by 8 blanks"));
        assertTrue (out.contains("<p>something at the bottom</p>"));
    }
    
    @Test
    public void testTitle() throws Exception {
        String[] args = {"-title", "alternate Title", "test.cpp", "test.cpp.html"};
        Code2HTML.main(args);
        String out = load("test.cpp.html");
        assertTrue (out.contains("a line started by 8 blanks"));
        assertTrue (out.contains("<title>alternate Title</title>"));
        assertTrue (out.contains("<h1>alternate Title</h1>"));
    }
    
}
